@@ -891,17 +891,27 @@ pyg_value_from_pyobject(GValue *value, PyObject *obj)
     case G_TYPE_STRING:
 	if (obj == Py_None) {
 	    g_value_set_string(value, NULL);
-#if PY_VERSION_HEX < 0x03000000
-	} else if (PyString_Check(obj)) {
-	    g_value_set_string(value, PyString_AsString(obj));
-#endif
-	} else if (PyUnicode_Check(obj)) {
-	    tmp = PyUnicode_AsUTF8String(obj);
-	    g_value_set_string(value, PYGLIB_PyBytes_AsString(tmp));
-	    Py_DECREF(tmp);
 	} else {
-	    PyErr_Clear();
-	    return -1;
+	    PyObject* tmp_str = PyObject_Str(obj);
+	    if (tmp_str == NULL) {
+	        PyErr_Clear();
+	        if (PyUnicode_Check(obj)) {
+	            tmp = PyUnicode_AsUTF8String(obj);
+	            g_value_set_string(value, PYGLIB_PyBytes_AsString(tmp));
+	            Py_DECREF(tmp);
+	        } else {
+	            return -1;
+	        }
+	    } else {
+#if PY_VERSION_HEX < 0x03000000
+	       g_value_set_string(value, PyString_AsString(obj));
+#else
+	       tmp = PyUnicode_AsUTF8String(obj);
+	       g_value_set_string(value, PyBytes_AsString(tmp));
+	       Py_DECREF(tmp);
+#endif
+	    }
+	    Py_XDECREF(tmp_str);
 	}
 	break;
     case G_TYPE_POINTER:
@@ -1012,7 +1022,7 @@ pyg_value_as_pyobject(const GValue *value, gboolean copy_boxed)
     switch (G_TYPE_FUNDAMENTAL(G_VALUE_TYPE(value))) {
     case G_TYPE_INTERFACE:
 	if (g_type_is_a(G_VALUE_TYPE(value), G_TYPE_OBJECT))
-	    return pygobject_new(g_value_get_object(value));
+	    return pygobject_new_sunk(g_value_get_object(value));
 	else
 	    break;
     case G_TYPE_CHAR: {
@@ -1021,7 +1031,7 @@ pyg_value_as_pyobject(const GValue *value, gboolean copy_boxed)
     }
     case G_TYPE_UCHAR: {
 	guint8 val = g_value_get_uchar(value);
-	return PYGLIB_PyUnicode_FromStringAndSize((char *)&val, 1);
+	return PYGLIB_PyBytes_FromStringAndSize((char *)&val, 1);
     }
     case G_TYPE_BOOLEAN: {
 	return PyBool_FromLong(g_value_get_boolean(value));
@@ -1130,7 +1140,7 @@ pyg_value_as_pyobject(const GValue *value, gboolean copy_boxed)
     case G_TYPE_PARAM:
 	return pyg_param_spec_new(g_value_get_param(value));
     case G_TYPE_OBJECT:
-	return pygobject_new(g_value_get_object(value));
+	return pygobject_new_sunk(g_value_get_object(value));
     default:
 	{
 	    PyGTypeMarshal *bm;
@@ -1326,7 +1336,7 @@ pyg_signal_class_closure_marshal(GClosure *closure,
     g_return_if_fail(object != NULL && G_IS_OBJECT(object));
 
     /* get the wrapper for this object */
-    object_wrapper = pygobject_new(object);
+    object_wrapper = pygobject_new_sunk(object);
     g_return_if_fail(object_wrapper != NULL);
 
     /* construct method name for this class closure */
